from http import HTTPStatus
from typing import Any, Dict, Optional

import httpx

from ... import errors
from ...client import Client
from ...types import Response


def _get_kwargs(
    parent_note_id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/refresh-note-ordering/{parentNoteId}".format(client.base_url, parentNoteId=parent_note_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.NO_CONTENT:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    parent_note_id: str,
    *,
    client: Client,
) -> Response[Any]:
    r"""notePositions in branches are not automatically pushed to connected clients and need a specific
    instruction.  If you want your changes to be in effect immediately, call this service after setting
    branches' notePosition.  Note that you need to supply \"parentNoteId\" of branch(es) with changed
    positions.

    Args:
        parent_note_id (str):  Example: evnnmvHTCgIn.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        parent_note_id=parent_note_id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    parent_note_id: str,
    *,
    client: Client,
) -> Response[Any]:
    r"""notePositions in branches are not automatically pushed to connected clients and need a specific
    instruction.  If you want your changes to be in effect immediately, call this service after setting
    branches' notePosition.  Note that you need to supply \"parentNoteId\" of branch(es) with changed
    positions.

    Args:
        parent_note_id (str):  Example: evnnmvHTCgIn.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        parent_note_id=parent_note_id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)
